@@ -58,6 +58,7 @@ struct _EphyEmbedShellPrivate
   GList *web_extensions;
   guint web_extensions_page_created_signal_id;
   guint web_extensions_form_auth_save_signal_id;
+  guint web_extensions_remove_from_overview_signal_id;
 };
 
 enum
@@ -160,6 +161,26 @@ web_extension_page_created (GDBusConnection *connection,
   g_signal_emit (shell, signals[PAGE_CREATED], 0, page_id, web_extension);
 }
 
+static void
+web_extension_remove_from_overview (GDBusConnection *connection,
+                                    const char *sender_name,
+                                    const char *object_path,
+                                    const char *interface_name,
+                                    const char *signal_name,
+                                    GVariant *parameters,
+                                    EphyEmbedShell *shell)
+{
+  GtkTreeIter iter;
+  EphyFrecentStore *store;
+  const char *url_to_remove;
+
+  g_variant_get (parameters, "(&s)", &url_to_remove);
+  store = ephy_embed_shell_get_frecent_store (ephy_embed_shell_get_default ());
+  if (ephy_overview_store_find_url (EPHY_OVERVIEW_STORE(store), url_to_remove, &iter)) {
+	  ephy_frecent_store_set_hidden(store, &iter);
+  }
+}
+
 static void
 web_extension_destroyed (EphyEmbedShell *shell,
                          GObject *web_extension)
@@ -191,6 +212,117 @@ ephy_embed_shell_unwatch_web_extension (EphyWebExtensionProxy *web_extension,
   g_object_weak_unref (G_OBJECT (web_extension), (GWeakNotify)web_extension_destroyed, shell);
 }
 
+static void
+history_service_query_urls_cb (EphyHistoryService *service,
+                               gboolean success,
+                               GList *urls,
+                               EphyEmbedShell *shell)
+{
+  GList *l;
+
+  if (!success)
+    return;
+
+  for (l = shell->priv->web_extensions; l; l = g_list_next (l)) {
+    EphyWebExtensionProxy *web_extension = (EphyWebExtensionProxy *)l->data;
+
+    ephy_web_extension_proxy_history_set_urls (web_extension, urls);
+  }
+}
+
+static void
+history_service_urls_visited_cb (EphyHistoryService *history,
+                                 EphyEmbedShell *shell)
+{
+  EphyHistoryQuery *query;
+
+  query = ephy_history_query_new ();
+  query->sort_type = EPHY_HISTORY_SORT_MOST_VISITED;
+  query->limit = ephy_frecent_store_get_history_length (ephy_embed_shell_get_frecent_store (shell));
+  query->ignore_hidden = TRUE;
+
+  ephy_history_service_query_urls (history, query, NULL,
+                                   (EphyHistoryJobCallback) history_service_query_urls_cb,
+                                   shell);
+  ephy_history_query_free (query);
+}
+
+static void
+history_service_url_title_changed_cb (EphyHistoryService *service,
+                                      const char *url,
+                                      const char *title,
+                                      EphyEmbedShell *shell)
+{
+  GList *l;
+
+  for (l = shell->priv->web_extensions; l; l = g_list_next (l)) {
+    EphyWebExtensionProxy *web_extension = (EphyWebExtensionProxy *)l->data;
+
+    ephy_web_extension_proxy_history_set_url_title (web_extension, url, title);
+  }
+}
+
+static void
+history_service_url_deleted_cb (EphyHistoryService *service,
+                                const char *url,
+                                EphyEmbedShell *shell)
+{
+  GList *l;
+
+  for (l = shell->priv->web_extensions; l; l = g_list_next (l)) {
+    EphyWebExtensionProxy *web_extension = (EphyWebExtensionProxy *)l->data;
+
+    ephy_web_extension_proxy_history_delete_url (web_extension, url);
+  }
+}
+
+static void
+history_service_host_deleted_cb (EphyHistoryService *service,
+                                 const char *deleted_url,
+                                 EphyEmbedShell *shell)
+{
+  GList *l;
+  SoupURI *deleted_uri;
+
+  deleted_uri = soup_uri_new (deleted_url);
+
+  for (l = shell->priv->web_extensions; l; l = g_list_next (l)) {
+    EphyWebExtensionProxy *web_extension = (EphyWebExtensionProxy *)l->data;
+
+    ephy_web_extension_proxy_history_delete_host (web_extension, soup_uri_get_host (deleted_uri));
+  }
+
+  soup_uri_free (deleted_uri);
+}
+
+static void
+history_service_cleared_cb (EphyHistoryService *service,
+                            EphyEmbedShell *shell)
+{
+  GList *l;
+
+  for (l = shell->priv->web_extensions; l; l = g_list_next (l)) {
+    EphyWebExtensionProxy *web_extension = (EphyWebExtensionProxy *)l->data;
+
+    ephy_web_extension_proxy_history_clear (web_extension);
+  }
+}
+
+static void
+frecent_store_snapshot_saved_cb (EphyOverviewStore *store,
+                                 const char *url,
+                                 const char *path,
+                                 EphyEmbedShell *shell)
+{
+  GList *l;
+
+  for (l = shell->priv->web_extensions; l; l = g_list_next (l)) {
+    EphyWebExtensionProxy *web_extension = (EphyWebExtensionProxy *)l->data;
+
+    ephy_web_extension_proxy_history_set_url_thumbnail (web_extension, url, path);
+  }
+}
+
 /**
  * ephy_embed_shell_get_global_history_service:
  * @shell: the #EphyEmbedShell
@@ -209,6 +341,24 @@ ephy_embed_shell_get_global_history_service (EphyEmbedShell *shell)
     shell->priv->global_history_service = ephy_history_service_new (filename, FALSE);
     g_free (filename);
     g_return_val_if_fail (shell->priv->global_history_service, NULL);
+    g_signal_connect (shell->priv->global_history_service, "urls-visited",
+                      G_CALLBACK (history_service_urls_visited_cb),
+                      shell);
+    g_signal_connect (shell->priv->global_history_service, "url-title-changed",
+                      G_CALLBACK (history_service_url_title_changed_cb),
+                      shell);
+    g_signal_connect (shell->priv->global_history_service, "url-deleted",
+                      G_CALLBACK (history_service_url_deleted_cb),
+                      shell);
+    g_signal_connect (shell->priv->global_history_service, "host-deleted",
+                      G_CALLBACK (history_service_host_deleted_cb),
+                      shell);
+    g_signal_connect (shell->priv->global_history_service, "cleared",
+                      G_CALLBACK (history_service_cleared_cb),
+                      shell);
+    g_signal_connect (ephy_embed_shell_get_frecent_store (shell), "snapshot-saved",
+                      G_CALLBACK (frecent_store_snapshot_saved_cb),
+                      shell);
   }
 
   return G_OBJECT (shell->priv->global_history_service);
@@ -245,23 +395,19 @@ EphyFrecentStore *
 ephy_embed_shell_get_frecent_store (EphyEmbedShell *shell)
 {
   GdkPixbuf *default_icon;
-  GdkPixbuf *frame;
 
   g_return_val_if_fail (EPHY_IS_EMBED_SHELL (shell), NULL);
 
   if (shell->priv->frecent_store == NULL) {
     shell->priv->frecent_store = ephy_frecent_store_new ();
     default_icon = ephy_embed_shell_get_overview_icon ("missing-thumbnail.png");
-    frame = ephy_embed_shell_get_overview_icon ("thumbnail-frame.png");
     g_object_set (shell->priv->frecent_store,
                   "history-service",
                   ephy_embed_shell_get_global_history_service (shell),
                   "history-length", 10,
                   "default-icon", default_icon,
-                  "icon-frame", frame,
                   NULL);
     g_object_unref (default_icon);
-    g_object_unref (frame);
   }
 
   return shell->priv->frecent_store;
@@ -356,6 +502,17 @@ ephy_embed_shell_setup_web_extensions_connection (EphyEmbedShell *shell)
                                         (GDBusSignalCallback)web_extension_form_auth_save_requested,
                                         shell,
                                         NULL);
+  shell->priv->web_extensions_remove_from_overview_signal_id =
+    g_dbus_connection_signal_subscribe (shell->priv->bus,
+                                        NULL,
+                                        EPHY_WEB_EXTENSION_INTERFACE,
+                                        "RemoveItemFromOverview",
+                                        EPHY_WEB_EXTENSION_OBJECT_PATH,
+                                        NULL,
+                                        G_DBUS_SIGNAL_FLAGS_NONE,
+                                        (GDBusSignalCallback)web_extension_remove_from_overview,
+                                        shell,
+                                        NULL);
 }
 
 static void
@@ -370,7 +527,7 @@ ephy_embed_shell_setup_process_model (EphyEmbedShell *shell,
     webkit_web_context_set_process_model (web_context, WEBKIT_PROCESS_MODEL_SHARED_SECONDARY_PROCESS);
     break;
   case EPHY_PREFS_PROCESS_MODEL_ONE_SECONDARY_PROCESS_PER_WEB_VIEW:
-    webkit_web_context_set_process_model (web_context, WEBKIT_PROCESS_MODEL_ONE_SECONDARY_PROCESS_PER_WEB_VIEW);
+    webkit_web_context_set_process_model (web_context, WEBKIT_PROCESS_MODEL_MULTIPLE_SECONDARY_PROCESSES);
     break;
   }
 }
@@ -413,6 +570,10 @@ ephy_embed_shell_startup (GApplication* application)
                                           (WebKitURISchemeRequestCallback)about_request_cb,
                                           shell, NULL);
 
+  /* Register about scheme as local so that it can contain file resources */
+  webkit_security_manager_register_uri_scheme_as_local (webkit_web_context_get_security_manager (web_context),
+                                                        EPHY_ABOUT_SCHEME);
+
   /* Store cookies in moz-compatible SQLite format */
   cookie_manager = webkit_web_context_get_cookie_manager (web_context);
   filename = g_build_filename (ephy_dot_dir (), "cookies.sqlite", NULL);
@@ -445,6 +606,11 @@ ephy_embed_shell_shutdown (GApplication* application)
     priv->web_extensions_form_auth_save_signal_id = 0;
   }
 
+  if (priv->web_extensions_remove_from_overview_signal_id > 0) {
+    g_dbus_connection_signal_unsubscribe (priv->bus, priv->web_extensions_remove_from_overview_signal_id);
+    priv->web_extensions_remove_from_overview_signal_id = 0;
+  }
+
   g_list_foreach (priv->web_extensions, (GFunc)ephy_embed_shell_unwatch_web_extension, application);
 
   ephy_embed_prefs_shutdown ();
