@@ -66,8 +66,6 @@ struct _EphyEmbedPrivate
 
   WebKitURIRequest *delayed_request;
 
-  GtkWidget *overview;
-  guint overview_mode : 1;
   GSList *messages;
   GSList *keys;
 
@@ -88,7 +86,7 @@ struct _EphyEmbedPrivate
 enum
 {
   PROP_0,
-  PROP_OVERVIEW_MODE,
+  PROP_WEB_VIEW,
 };
 
 G_DEFINE_TYPE (EphyEmbed, ephy_embed, GTK_TYPE_BOX)
@@ -247,20 +245,8 @@ load_changed_cb (WebKitWebView *web_view,
                  WebKitLoadEvent load_event,
                  EphyEmbed *embed)
 {
-  switch (load_event) {
-  case WEBKIT_LOAD_STARTED: {
-    const char *uri;
-
-    uri = webkit_web_view_get_uri (web_view);
-    ephy_embed_set_overview_mode (embed, g_strcmp0 (uri, "ephy-about:overview") == 0);
-    break;
-  }
-  case WEBKIT_LOAD_COMMITTED:
+  if (load_event == WEBKIT_LOAD_COMMITTED)
     ephy_embed_destroy_top_widgets (embed);
-    break;
-  default:
-    break;
-  }
 }
 
 static void
@@ -290,15 +276,17 @@ fullscreen_message_label_hide (EphyEmbed *embed)
 void
 ephy_embed_entering_fullscreen (EphyEmbed *embed)
 {
-  gtk_widget_show (embed->priv->fullscreen_message_label);
+  if (!g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN, EPHY_PREFS_LOCKDOWN_FULLSCREEN)) {
+    gtk_widget_show (embed->priv->fullscreen_message_label);
 
-  if (embed->priv->fullscreen_message_id)
-    g_source_remove (embed->priv->fullscreen_message_id);
+    if (embed->priv->fullscreen_message_id)
+      g_source_remove (embed->priv->fullscreen_message_id);
 
-  embed->priv->fullscreen_message_id = g_timeout_add_seconds (5,
-                                                              (GSourceFunc)fullscreen_message_label_hide,
-                                                              embed);
-  g_source_set_name_by_id (embed->priv->fullscreen_message_id, "[epiphany] fullscreen_message_label_hide");
+    embed->priv->fullscreen_message_id = g_timeout_add_seconds (5,
+                                                                (GSourceFunc)fullscreen_message_label_hide,
+                                                                embed);
+    g_source_set_name_by_id (embed->priv->fullscreen_message_id, "[epiphany] fullscreen_message_label_hide");
+  }
 }
 
 void
@@ -394,8 +382,8 @@ ephy_embed_set_property (GObject *object,
 
   switch (prop_id)
   {
-  case PROP_OVERVIEW_MODE:
-    ephy_embed_set_overview_mode (embed, g_value_get_boolean (value));
+  case PROP_WEB_VIEW:
+    embed->priv->web_view = g_value_get_object (value);
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -413,8 +401,8 @@ ephy_embed_get_property (GObject *object,
 
   switch (prop_id)
   {
-  case PROP_OVERVIEW_MODE:
-    g_value_set_boolean (value, ephy_embed_get_overview_mode (embed));
+  case PROP_WEB_VIEW:
+    g_value_set_object (value, ephy_embed_get_web_view (embed));
     break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -446,18 +434,13 @@ ephy_embed_class_init (EphyEmbedClass *klass)
   object_class->get_property = ephy_embed_get_property;
   widget_class->grab_focus = ephy_embed_grab_focus;
 
-/**
- * EphyEmbed:overview-mode:
- *
- * If %TRUE activates the overview mode in this #EphyEmbed.
- **/
   g_object_class_install_property (object_class,
-                                   PROP_OVERVIEW_MODE,
-                                   g_param_spec_boolean ("overview-mode",
-                                                         "Overview mode",
-                                                         "Whether the embed is showing the overview",
-                                                         FALSE,
-                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                   PROP_WEB_VIEW,
+                                   g_param_spec_object ("web-view",
+                                                        "Web View",
+                                                        "The WebView contained in the embed",
+                                                        EPHY_TYPE_WEB_VIEW,
+                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
   g_type_class_add_private (G_OBJECT_CLASS (klass), sizeof(EphyEmbedPrivate));
 }
@@ -636,7 +619,6 @@ ephy_embed_constructed (GObject *object)
   EphyEmbedPrivate *priv = embed->priv;
   EphyEmbedShell *shell = ephy_embed_shell_get_default ();
   GtkWidget *paned;
-  WebKitWebView *web_view;
   WebKitWebInspector *inspector;
   GtkWidget *overlay;
 
@@ -647,26 +629,12 @@ ephy_embed_constructed (GObject *object)
                     G_CALLBACK (ephy_embed_mapped_cb), NULL);
 
   /* Skeleton */
-  web_view = WEBKIT_WEB_VIEW (ephy_web_view_new ());
   overlay = gtk_overlay_new ();
 
   gtk_widget_add_events (overlay, 
                          GDK_ENTER_NOTIFY_MASK |
                          GDK_LEAVE_NOTIFY_MASK);
-  gtk_container_add (GTK_CONTAINER (overlay), GTK_WIDGET (web_view));
-
-  /* The overview. In incognito mode we don't use it. */
-  if (ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) !=
-      EPHY_EMBED_SHELL_MODE_INCOGNITO) {
-    priv->overview = ephy_overview_new ();
-    gtk_widget_set_halign (priv->overview, GTK_ALIGN_FILL);
-    gtk_widget_set_valign (priv->overview, GTK_ALIGN_FILL);
-    gtk_overlay_add_overlay (GTK_OVERLAY (overlay), priv->overview);
-
-    g_object_bind_property (embed, "overview-mode",
-                            priv->overview, "visible",
-                            G_BINDING_SYNC_CREATE);
-  }
+  gtk_container_add (GTK_CONTAINER (overlay), GTK_WIDGET (priv->web_view));
 
   /* Floating message popup for fullscreen mode. */
   priv->fullscreen_message_label = gtk_label_new (NULL);
@@ -703,8 +671,7 @@ ephy_embed_constructed (GObject *object)
 
   paned = GTK_WIDGET (priv->paned);
 
-  priv->web_view = web_view;
-  priv->progress_update_handler_id = g_signal_connect (web_view, "notify::estimated-load-progress",
+  priv->progress_update_handler_id = g_signal_connect (priv->web_view, "notify::estimated-load-progress",
                                                        G_CALLBACK (progress_update), object);
   gtk_paned_pack1 (GTK_PANED (paned), GTK_WIDGET (overlay),
                    TRUE, FALSE);
@@ -715,21 +682,21 @@ ephy_embed_constructed (GObject *object)
   gtk_box_pack_start (GTK_BOX (embed), paned, TRUE, TRUE, 0);
 
   gtk_widget_show (GTK_WIDGET (priv->top_widgets_vbox));
-  gtk_widget_show (GTK_WIDGET (web_view));
+  gtk_widget_show (GTK_WIDGET (priv->web_view));
   gtk_widget_show_all (paned);
 
-  g_object_connect (web_view,
+  g_object_connect (priv->web_view,
                     "signal::load-changed", G_CALLBACK (load_changed_cb), embed,
                     "signal::enter-fullscreen", G_CALLBACK (entering_fullscreen_cb), embed,
                     "signal::leave-fullscreen", G_CALLBACK (leaving_fullscreen_cb), embed,
                     NULL);
 
-  priv->status_handler_id = g_signal_connect (web_view, "notify::status-message",
+  priv->status_handler_id = g_signal_connect (priv->web_view, "notify::status-message",
                                               G_CALLBACK (status_message_notify_cb),
                                               embed);
 
   /* The inspector */
-  inspector = webkit_web_view_get_inspector (web_view);
+  inspector = webkit_web_view_get_inspector (priv->web_view);
 
   g_signal_connect (inspector, "attach",
                     G_CALLBACK (ephy_embed_attach_inspector_cb),
@@ -838,31 +805,6 @@ ephy_embed_remove_top_widget (EphyEmbed *embed, GtkWidget *widget)
                         GTK_WIDGET (widget));
 }
 
-void
-ephy_embed_set_overview_mode (EphyEmbed *embed, gboolean overview_mode)
-{
-  EphyEmbedPrivate *priv;
-
-  g_return_if_fail (EPHY_IS_EMBED (embed));
-
-  priv = embed->priv;
-
-  if (priv->overview_mode == overview_mode)
-    return;
-
-  priv->overview_mode = overview_mode;
-
-  g_object_notify (G_OBJECT (embed), "overview-mode");
-}
-
-gboolean
-ephy_embed_get_overview_mode (EphyEmbed *embed)
-{
-  g_return_val_if_fail (EPHY_IS_EMBED (embed), FALSE);
-
-  return embed->priv->overview_mode;
-}
-
 /**
  * ephy_embed_set_delayed_load_request:
  * @embed: a #EphyEmbed
@@ -899,18 +841,3 @@ ephy_embed_has_load_pending (EphyEmbed *embed)
   return !!embed->priv->delayed_request;
 }
 
-/**
- * ephy_embed_get_overview:
- * @embed: a #EphyEmbed
- *
- * Gets the #EphyOverview in this @embed
- *
- * Returns: (transfer none): the overview widget
- **/
-EphyOverview *
-ephy_embed_get_overview (EphyEmbed *embed)
-{
-  g_return_val_if_fail (EPHY_IS_EMBED (embed), NULL);
-
-  return EPHY_OVERVIEW (embed->priv->overview);
-}
