@@ -37,7 +37,6 @@
 #include "ephy-file-monitor.h"
 #include "ephy-form-auth-data.h"
 #include "ephy-history-service.h"
-#include "ephy-overview.h"
 #include "ephy-prefs.h"
 #include "ephy-settings.h"
 #include "ephy-string.h"
@@ -739,7 +738,7 @@ get_title_from_address (const char *address)
     return g_strdup (address + 7);
   else if (!strcmp (address, EPHY_ABOUT_SCHEME":overview") ||
            !strcmp (address, "about:overview"))
-    return g_strdup (EPHY_OVERVIEW_TITLE);
+    return g_strdup (_("Most Visited"));
   else
     return ephy_string_get_host_name (address);
 }
@@ -1521,8 +1520,6 @@ on_snapshot_ready (WebKitWebView *webview,
                    GtkTreeRowReference *ref)
 {
   GtkTreeModel *model;
-  GtkTreePath *path;
-  GtkTreeIter iter;
   cairo_surface_t *surface;
   GError *error = NULL;
 
@@ -1534,11 +1531,7 @@ on_snapshot_ready (WebKitWebView *webview,
   }
 
   model = gtk_tree_row_reference_get_model (ref);
-  path = gtk_tree_row_reference_get_path (ref);
-  gtk_tree_model_get_iter (model, &iter, path);
-  gtk_tree_path_free (path);
-
-  ephy_overview_store_set_snapshot (EPHY_OVERVIEW_STORE (model), &iter, surface,
+  ephy_overview_store_set_snapshot (EPHY_OVERVIEW_STORE (model), ref, surface,
                                     webkit_web_view_get_favicon (webview));
   cairo_surface_destroy (surface);
 }
@@ -2022,9 +2015,9 @@ ephy_web_view_init (EphyWebView *web_view)
                     G_CALLBACK (ge_popup_blocked_cb),
                     NULL);
 
-  g_signal_connect (ephy_embed_shell_get_default (), "page-created",
-                    G_CALLBACK (page_created_cb),
-                    web_view);
+  g_signal_connect_object (ephy_embed_shell_get_default (), "page-created",
+                           G_CALLBACK (page_created_cb),
+                           web_view, 0);
 }
 
 /**
@@ -2043,6 +2036,15 @@ ephy_web_view_new (void)
                        NULL);
 }
 
+GtkWidget *
+ephy_web_view_new_with_related_view (WebKitWebView *related_view)
+{
+  return g_object_new (EPHY_TYPE_WEB_VIEW,
+                       "related-view", related_view,
+                       "group", ephy_embed_prefs_get_web_view_group (),
+                       NULL);
+}
+
 static gboolean
 is_public_domain (EphyWebView *view, const char *url)
 {
@@ -2261,24 +2263,6 @@ ephy_web_view_load_url (EphyWebView *view,
   g_free (effective_url);
 }
 
-/**
- * ephy_web_view_copy_back_history:
- * @source: the #EphyWebView from which to get the back history
- * @dest: the #EphyWebView to copy the history to
- *
- * Sets the back history (up to the current item) of @source as the
- * back history of @dest.
- *
- * Useful to keep the history when opening links in new tabs or
- * windows.
- **/
-void
-ephy_web_view_copy_back_history (EphyWebView *source,
-                                 EphyWebView *dest)
-{
-  /* TODO: WebKit2, BackForwardList */
-}
-
 /**
  * ephy_web_view_get_is_blank:
  * @view: an #EphyWebView
@@ -2293,6 +2277,18 @@ ephy_web_view_get_is_blank (EphyWebView *view)
   return view->priv->is_blank;
 }
 
+gboolean
+ephy_web_view_is_overview (EphyWebView *view)
+{
+  EphyWebViewPrivate *priv = view->priv;
+
+  if (!priv->address)
+    return FALSE;
+
+  return (!strcmp (priv->address, EPHY_ABOUT_SCHEME":overview") ||
+          !strcmp (priv->address, "about:overview"));
+}
+
 /**
  * ephy_web_view_get_address:
  * @view: an #EphyWebView
