@@ -151,6 +151,7 @@ ephy_history_service_add_url_row (EphyHistoryService *self, EphyHistoryURL *url)
       ephy_sqlite_statement_bind_int (statement, 5, url->host->id, &error) == FALSE) {
     g_error ("Could not insert URL into urls table: %s", error->message);
     g_error_free (error);
+    g_object_unref (statement);
     return;
   }
 
@@ -193,6 +194,7 @@ ephy_history_service_update_url_row (EphyHistoryService *self, EphyHistoryURL *u
       ephy_sqlite_statement_bind_int (statement, 6, url->id, &error) == FALSE) {
     g_error ("Could not modify URL in urls table: %s", error->message);
     g_error_free (error);
+    g_object_unref (statement);
     return;
   }
 
@@ -272,12 +274,18 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
   statement_str = g_string_append (statement_str, "1 ");
 
   switch (query->sort_type) {
-  case EPHY_HISTORY_SORT_MV:
+  case EPHY_HISTORY_SORT_MOST_VISITED:
     statement_str = g_string_append (statement_str, "ORDER BY urls.visit_count DESC ");
     break;
-  case EPHY_HISTORY_SORT_LV:
+  case EPHY_HISTORY_SORT_LEAST_VISITED:
     statement_str = g_string_append (statement_str, "ORDER BY urls.visit_count ");
     break;
+  case EPHY_HISTORY_SORT_MOST_RECENTLY_VISITED:
+    statement_str = g_string_append (statement_str, "ORDER BY urls.last_visit_time DESC ");
+    break;
+  case EPHY_HISTORY_SORT_LEAST_RECENTLY_VISITED:
+    statement_str = g_string_append (statement_str, "ORDER BY urls.last_visit_time ");
+    break;
   default:
     g_warning ("We don't support this sorting method yet.");
   }
@@ -293,7 +301,6 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
   if (error) {
     g_error ("Could not build urls table query statement: %s", error->message);
     g_error_free (error);
-    g_object_unref (statement);
     return NULL;
   }
 
@@ -330,7 +337,7 @@ ephy_history_service_find_url_rows (EphyHistoryService *self, EphyHistoryQuery *
       g_free (string);
       return NULL;
     }
-    if (ephy_sqlite_statement_bind_string (statement, i++, string, &error) == FALSE) {
+    if (ephy_sqlite_statement_bind_string (statement, i++, string + 2, &error) == FALSE) {
       g_error ("Could not build urls table query statement: %s", error->message);
       g_error_free (error);
       g_object_unref (statement);
@@ -390,7 +397,6 @@ ephy_history_service_delete_url (EphyHistoryService *self, EphyHistoryURL *url)
   if (error) {
     g_error ("Could not build urls table query statement: %s", error->message);
     g_error_free (error);
-    g_object_unref (statement);
     return;
   }
 
