@@ -21,7 +21,6 @@
 #include "config.h"
 #include "ephy-history-service.h"
 #include "ephy-overview-store.h"
-#include "ephy-removable-pixbuf-renderer.h"
 #include "ephy-snapshot-service.h"
 #include "ephy-widgets-type-builtins.h"
 
@@ -34,17 +33,23 @@ struct _EphyOverviewStorePrivate
 {
   EphyHistoryService *history_service;
   GdkPixbuf *default_icon;
-  GdkPixbuf *icon_frame;
 };
 
 enum
 {
   PROP_0,
   PROP_HISTORY_SERVICE,
-  PROP_DEFAULT_ICON,
-  PROP_ICON_FRAME,
+  PROP_DEFAULT_ICON
 };
 
+enum {
+  SNAPSHOT_SAVED,
+
+  LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL];
+
 G_DEFINE_TYPE (EphyOverviewStore, ephy_overview_store, GTK_TYPE_LIST_STORE)
 
 static void
@@ -65,10 +70,6 @@ ephy_overview_store_set_property (GObject *object,
     ephy_overview_store_set_default_icon (store,
                                           g_value_get_object (value));
     break;
-  case PROP_ICON_FRAME:
-    ephy_overview_store_set_icon_frame (store,
-                                        g_value_get_object (value));
-    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     break;
@@ -91,9 +92,6 @@ ephy_overview_store_get_property (GObject *object,
   case PROP_DEFAULT_ICON:
     g_value_set_object (value, store->priv->default_icon);
     break;
-  case PROP_ICON_FRAME:
-    g_value_set_object (value, store->priv->icon_frame);
-    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
     break;
@@ -109,8 +107,6 @@ ephy_overview_store_dispose (GObject *object)
     g_clear_object (&priv->history_service);
   if (priv->default_icon)
     g_clear_object (&priv->default_icon);
-  if (priv->icon_frame)
-    g_clear_object (&priv->icon_frame);
 
   G_OBJECT_CLASS (ephy_overview_store_parent_class)->dispose (object);
 }
@@ -140,13 +136,15 @@ ephy_overview_store_class_init (EphyOverviewStoreClass *klass)
                                                         GDK_TYPE_PIXBUF,
                                                         G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
-  g_object_class_install_property (object_class,
-                                   PROP_ICON_FRAME,
-                                   g_param_spec_object ("icon-frame",
-                                                        "Icon frame",
-                                                        "Frame to display around icons",
-                                                        GDK_TYPE_PIXBUF,
-                                                        G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+  signals[SNAPSHOT_SAVED] =
+    g_signal_new ("snapshot-saved",
+                  EPHY_TYPE_OVERVIEW_STORE,
+                  G_SIGNAL_RUN_LAST,
+                  0, NULL, NULL,
+                  g_cclosure_marshal_generic,
+                  G_TYPE_NONE, 2,
+                  G_TYPE_STRING,
+                  G_TYPE_STRING);
 
   g_type_class_add_private (object_class, sizeof(EphyOverviewStorePrivate));
 }
@@ -165,7 +163,7 @@ ephy_overview_store_init (EphyOverviewStore *self)
   types[EPHY_OVERVIEW_STORE_SELECTED] = G_TYPE_BOOLEAN;
   types[EPHY_OVERVIEW_STORE_SNAPSHOT_CANCELLABLE] = G_TYPE_CANCELLABLE;
   types[EPHY_OVERVIEW_STORE_SNAPSHOT_MTIME] = G_TYPE_INT;
-  types[EPHY_OVERVIEW_STORE_CLOSE_BUTTON_RENDER_POLICY] = EPHY_TYPE_REMOVABLE_PIXBUF_RENDER_POLICY;
+  types[EPHY_OVERVIEW_STORE_SNAPSHOT_PATH] = G_TYPE_STRING;
 
   gtk_list_store_set_column_types (GTK_LIST_STORE (self),
                                    EPHY_OVERVIEW_STORE_NCOLS, types);
@@ -194,43 +192,34 @@ peek_context_free (PeekContext *ctx)
   g_slice_free (PeekContext, ctx);
 }
 
-static GdkPixbuf *
-ephy_overview_store_add_frame (EphyOverviewStore *store,
-                               GdkPixbuf *snapshot)
-{
-  GdkPixbuf *framed;
-
-  if (store->priv->icon_frame) {
-    framed = gdk_pixbuf_copy (store->priv->icon_frame);
-    gdk_pixbuf_copy_area (snapshot, 0, 0,
-                          gdk_pixbuf_get_width (snapshot),
-                          gdk_pixbuf_get_height (snapshot),
-                          framed, 10, 9);
-  } else
-    framed = g_object_ref (snapshot);
-
-  return framed;
-}
-
 static void
 ephy_overview_store_set_snapshot_internal (EphyOverviewStore *store,
                                            GtkTreeIter *iter,
                                            GdkPixbuf *snapshot,
+                                           char *path,
                                            int mtime)
 {
-  GdkPixbuf *framed;
-
-  framed = ephy_overview_store_add_frame (store, snapshot);
   gtk_list_store_set (GTK_LIST_STORE (store), iter,
-                      EPHY_OVERVIEW_STORE_SNAPSHOT, framed,
+                      EPHY_OVERVIEW_STORE_SNAPSHOT, snapshot,
                       EPHY_OVERVIEW_STORE_SNAPSHOT_MTIME, mtime,
+                      EPHY_OVERVIEW_STORE_SNAPSHOT_PATH, path,
                       -1);
-  g_object_unref (framed);
+
+  if (path) {
+    char *url;
+
+    gtk_tree_model_get (GTK_TREE_MODEL (store), iter,
+                        EPHY_OVERVIEW_STORE_URI, &url,
+                        -1);
+    g_signal_emit (store, signals[SNAPSHOT_SAVED], 0, url, path);
+    g_free (url);
+  }
 }
 
 typedef struct {
   EphyHistoryURL *url;
   EphyHistoryService *history_service;
+  GtkTreeRowReference *ref;
 } ThumbnailTimeContext;
 
 static void
@@ -238,6 +227,25 @@ on_snapshot_saved_cb (EphySnapshotService *service,
                       GAsyncResult *res,
                       ThumbnailTimeContext *ctx)
 {
+  char *path;
+  GtkTreeModel *model;
+  GtkTreePath *tree_path;
+  GtkTreeIter iter;
+
+  path = ephy_snapshot_service_save_snapshot_finish (service, res, NULL);
+
+  model = gtk_tree_row_reference_get_model (ctx->ref);
+  tree_path = gtk_tree_row_reference_get_path (ctx->ref);
+  gtk_tree_model_get_iter (model, &iter, tree_path);
+  gtk_tree_path_free (tree_path);
+
+  gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+                      EPHY_OVERVIEW_STORE_SNAPSHOT_PATH, path,
+                      -1);
+  g_signal_emit (model, signals[SNAPSHOT_SAVED], 0, ctx->url->url, path);
+  g_free (path);
+  gtk_tree_row_reference_free (ctx->ref);
+
   ephy_history_service_set_url_thumbnail_time (ctx->history_service,
                                                ctx->url->url, ctx->url->thumbnail_time,
                                                NULL, NULL, NULL);
@@ -247,7 +255,7 @@ on_snapshot_saved_cb (EphySnapshotService *service,
 
 void
 ephy_overview_store_set_snapshot (EphyOverviewStore *store,
-                                  GtkTreeIter *iter,
+                                  GtkTreeRowReference *ref,
                                   cairo_surface_t *snapshot,
                                   cairo_surface_t *favicon)
 {
@@ -256,11 +264,17 @@ ephy_overview_store_set_snapshot (EphyOverviewStore *store,
   ThumbnailTimeContext *ctx;
   EphySnapshotService *snapshot_service;
   int mtime;
+  GtkTreePath *path;
+  GtkTreeIter iter;
+
+  path = gtk_tree_row_reference_get_path (ref);
+  gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path);
+  gtk_tree_path_free (path);
 
   mtime = time (NULL);
   pixbuf = ephy_snapshot_service_prepare_snapshot (snapshot, favicon);
-  ephy_overview_store_set_snapshot_internal (store, iter, pixbuf, mtime);
-  gtk_tree_model_get (GTK_TREE_MODEL (store), iter,
+  ephy_overview_store_set_snapshot_internal (store, &iter, pixbuf, NULL, mtime);
+  gtk_tree_model_get (GTK_TREE_MODEL (store), &iter,
                       EPHY_OVERVIEW_STORE_URI, &url,
                       -1);
 
@@ -268,6 +282,7 @@ ephy_overview_store_set_snapshot (EphyOverviewStore *store,
   ctx->url = ephy_history_url_new (url, NULL, 0, 0, 0);
   ctx->url->thumbnail_time = mtime;
   ctx->history_service = store->priv->history_service;
+  ctx->ref = gtk_tree_row_reference_copy (ref);
   g_free (url);
 
   snapshot_service = ephy_snapshot_service_get_default ();
@@ -295,18 +310,19 @@ ephy_overview_store_set_default_icon_internal (EphyOverviewStore *store,
 static void
 set_snapshot (EphyOverviewStore *store,
               GdkPixbuf *snapshot,
+              char *path,
               GtkTreeRowReference *ref,
               time_t timestamp)
 {
-  GtkTreePath *path;
+  GtkTreePath *tree_path;
   GtkTreeIter iter;
 
-  path = gtk_tree_row_reference_get_path (ref);
-  gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path);
-  gtk_tree_path_free (path);
+  tree_path = gtk_tree_row_reference_get_path (ref);
+  gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, tree_path);
+  gtk_tree_path_free (tree_path);
 
   if (snapshot)
-    ephy_overview_store_set_snapshot_internal (store, &iter, snapshot, timestamp);
+    ephy_overview_store_set_snapshot_internal (store, &iter, snapshot, path, timestamp);
   else
     ephy_overview_store_set_default_icon_internal (store, &iter, store->priv->default_icon);
 
@@ -321,14 +337,16 @@ on_snapshot_retrieved_cb (GObject *object,
                           PeekContext *ctx)
 {
   GdkPixbuf *snapshot;
+  char *path = NULL;
 
   snapshot = ephy_snapshot_service_get_snapshot_finish (EPHY_SNAPSHOT_SERVICE (object),
-                                                        res, NULL);
+                                                        res, &path, NULL);
 
   set_snapshot (EPHY_OVERVIEW_STORE (gtk_tree_row_reference_get_model (ctx->ref)),
-                snapshot, ctx->ref, ctx->timestamp);
+                snapshot, path, ctx->ref, ctx->timestamp);
   if (snapshot)
     g_object_unref (snapshot);
+  g_free (path);
 
   peek_context_free (ctx);
 }
@@ -339,14 +357,16 @@ on_snapshot_retrieved_for_url_cb (GObject *object,
                                   PeekContext *ctx)
 {
   GdkPixbuf *snapshot;
+  char *path = NULL;
 
   snapshot = ephy_snapshot_service_get_snapshot_for_url_finish (EPHY_SNAPSHOT_SERVICE (object),
-                                                                res, NULL);
+                                                                res, &path, NULL);
 
   set_snapshot (EPHY_OVERVIEW_STORE (gtk_tree_row_reference_get_model (ctx->ref)),
-                snapshot, ctx->ref, ctx->timestamp);
+                snapshot, path, ctx->ref, ctx->timestamp);
   if (snapshot)
     g_object_unref (snapshot);
+  g_free (path);
 
   peek_context_free (ctx);
 }
@@ -447,49 +467,21 @@ void
 ephy_overview_store_set_default_icon (EphyOverviewStore *store,
                                       GdkPixbuf *default_icon)
 {
-  GdkPixbuf *new_default_icon;
+  g_return_if_fail (EPHY_IS_OVERVIEW_STORE (store));
+  g_return_if_fail (GDK_IS_PIXBUF (default_icon));
 
   if (store->priv->default_icon)
     g_object_unref (store->priv->default_icon);
 
-  new_default_icon = ephy_overview_store_add_frame (store, default_icon);
-
   gtk_tree_model_foreach (GTK_TREE_MODEL (store),
                           (GtkTreeModelForeachFunc) set_default_icon_helper,
-                          new_default_icon);
+                          default_icon);
 
-  store->priv->default_icon = new_default_icon;
+  store->priv->default_icon = g_object_ref (default_icon);
 
   g_object_notify (G_OBJECT (store), "default-icon");
 }
 
-void
-ephy_overview_store_set_icon_frame (EphyOverviewStore *store,
-                                    GdkPixbuf *icon_frame)
-{
-  gboolean update_default = FALSE;
-  GdkPixbuf *old_default_icon;
-
-  if (store->priv->icon_frame == icon_frame)
-    return;
-
-  if (store->priv->icon_frame)
-    g_object_unref (store->priv->icon_frame);
-  else if (store->priv->default_icon)
-    update_default = TRUE;
-
-  store->priv->icon_frame = g_object_ref (icon_frame);
-
-  if (update_default) {
-    old_default_icon = g_object_ref (store->priv->default_icon);
-    ephy_overview_store_set_default_icon (store,
-                                          old_default_icon);
-    g_object_unref (old_default_icon);
-  }
-
-  g_object_notify (G_OBJECT (store), "icon-frame");
-}
-
 gboolean
 ephy_overview_store_needs_snapshot (EphyOverviewStore *store,
                                     GtkTreeIter *iter)
@@ -616,10 +608,6 @@ ephy_overview_store_animated_remove (EphyOverviewStore *store,
   gtk_tree_model_get_iter (GTK_TREE_MODEL (store), &iter, path);
   gtk_tree_path_free (path);
 
-  gtk_list_store_set (GTK_LIST_STORE (store), &iter,
-                      EPHY_OVERVIEW_STORE_CLOSE_BUTTON_RENDER_POLICY,
-                      EPHY_REMOVABLE_PIXBUF_RENDER_NEVER, -1);
-
   id = g_timeout_add (40, (GSourceFunc) animated_remove_func, ctx);
   g_source_set_name_by_id (id, "[epiphany] animated_remove_func");
 }
