@@ -34,7 +34,6 @@
 #include "ephy-gui.h"
 #include "ephy-history-service.h"
 #include "ephy-link.h"
-#include "ephy-overview.h"
 #include "ephy-shell.h"
 #include "ephy-type-builtins.h"
 #include "ephy-window.h"
@@ -117,7 +116,7 @@ action_activate (GtkAction *action)
                                   EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed))),
                                   NULL,
                                   NULL,
-                                  EPHY_NEW_TAB_IN_EXISTING_WINDOW | EPHY_NEW_TAB_DONT_COPY_HISTORY);
+                                  EPHY_NEW_TAB_IN_EXISTING_WINDOW);
 
       web_view = EPHY_GET_WEBKIT_WEB_VIEW_FROM_EMBED (embed);
       webkit_web_view_load_uri (web_view, back_uri);
@@ -329,8 +328,7 @@ middle_click_handle_on_history_menu_item (EphyNavigationHistoryAction *action,
                                   EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed))),
                                   embed,
                                   NULL,
-                                  EPHY_NEW_TAB_IN_EXISTING_WINDOW |
-                                  EPHY_NEW_TAB_DONT_COPY_HISTORY);
+                                  EPHY_NEW_TAB_IN_EXISTING_WINDOW);
   g_return_if_fail (new_embed != NULL);
 
   /* We manually set the back history instead of trusting
@@ -415,11 +413,6 @@ build_dropdown_menu (EphyNavigationHistoryAction *action)
     uri = webkit_back_forward_list_item_get_uri (hitem);
     title = g_strdup (webkit_back_forward_list_item_get_title (hitem));
 
-    /* The overview is not actually a webpage, so we need to hardcode
-     * this here. */
-    if (g_strcmp0 (uri, "ephy-about:overview") == 0)
-      title = g_strdup (EPHY_OVERVIEW_TITLE);
-
     if (title == NULL || g_strstrip (title)[0] == '\0')
       item = new_history_menu_item (EPHY_WEB_VIEW (web_view), uri, uri);
     else
