@@ -233,8 +233,7 @@ ephy_session_undo_close_tab (EphySession *session)
 	ClosedTab *tab;
 	EphyNewTabFlags flags = EPHY_NEW_TAB_OPEN_PAGE
 		| EPHY_NEW_TAB_PRESENT_WINDOW
-		| EPHY_NEW_TAB_JUMP
-		| EPHY_NEW_TAB_DONT_COPY_HISTORY;
+		| EPHY_NEW_TAB_JUMP;
 
 	g_return_if_fail (EPHY_IS_SESSION (session));
 
@@ -389,6 +388,7 @@ session_maybe_open_window (EphySession *session,
 	if (ephy_shell_get_n_windows (shell) == 0)
 	{
 		ephy_shell_new_tab_full (shell,
+					 NULL /* related view */,
 					 NULL /* window */, NULL /* tab */,
 					 NULL /* NetworkRequest */,
 					 EPHY_NEW_TAB_IN_NEW_WINDOW |
@@ -1309,7 +1309,10 @@ ephy_session_load_from_stream (EphySession *session,
 	session->priv->dont_save = TRUE;
 
 	task = g_task_new (session, cancellable, callback, user_data);
-	g_task_set_priority (task, G_PRIORITY_HIGH);
+	/* Use a priority lower than drawing events (HIGH_IDLE + 20) to make sure
+	 * the main window is shown as soon as possible at startup
+	 */
+	g_task_set_priority (task, G_PRIORITY_HIGH_IDLE + 30);
 
 	context = session_parser_context_new (session, user_time);
 	parser = g_markup_parse_context_new (&session_parser, 0, context, (GDestroyNotify)session_parser_context_free);
@@ -1451,7 +1454,10 @@ ephy_session_load (EphySession *session,
 	g_application_hold (G_APPLICATION (ephy_shell_get_default ()));
 
 	task = g_task_new (session, cancellable, callback, user_data);
-	g_task_set_priority (task, G_PRIORITY_HIGH);
+	/* Use a priority lower than drawing events (HIGH_IDLE + 20) to make sure
+	 * the main window is shown as soon as possible at startup
+	 */
+	g_task_set_priority (task, G_PRIORITY_HIGH_IDLE + 30);
 
 	save_to_file = get_session_file (filename);
 	data = load_async_data_new (user_time);
