@@ -658,6 +658,7 @@ ephy_shell_get_default (void)
  **/
 EphyEmbed *
 ephy_shell_new_tab_full (EphyShell *shell,
+                         WebKitWebView *related_view,
                          EphyWindow *parent_window,
                          EphyEmbed *previous_embed,
                          WebKitURIRequest *request,
@@ -673,7 +674,6 @@ ephy_shell_new_tab_full (EphyShell *shell,
   gboolean delayed_open_page = FALSE;
   gboolean jump_to = FALSE;
   gboolean active_is_blank = FALSE;
-  gboolean copy_history = TRUE;
   gboolean is_empty = FALSE;
   int position = -1;
 
@@ -688,7 +688,6 @@ ephy_shell_new_tab_full (EphyShell *shell,
   if (flags & EPHY_NEW_TAB_DELAYED_OPEN_PAGE) delayed_open_page = TRUE;
   if (flags & EPHY_NEW_TAB_IN_NEW_WINDOW) in_new_window = TRUE;
   if (flags & EPHY_NEW_TAB_IN_EXISTING_WINDOW) in_new_window = FALSE;
-  if (flags & EPHY_NEW_TAB_DONT_COPY_HISTORY) copy_history = FALSE;
   if (flags & EPHY_NEW_TAB_JUMP) jump_to = TRUE;
 
   fullscreen_lockdown = g_settings_get_boolean (EPHY_SETTINGS_LOCKDOWN,
@@ -722,7 +721,7 @@ ephy_shell_new_tab_full (EphyShell *shell,
     embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
     if (embed != NULL) {
       EphyWebView *view = ephy_embed_get_web_view (embed);
-      if ((ephy_web_view_get_is_blank (view) || ephy_embed_get_overview_mode (embed)) &&
+      if ((ephy_web_view_get_is_blank (view) || ephy_web_view_is_overview (view)) &&
           ephy_web_view_is_loading (view) == FALSE) {
         active_is_blank = TRUE;
       }
@@ -730,18 +729,20 @@ ephy_shell_new_tab_full (EphyShell *shell,
   }
 
   if (active_is_blank == FALSE) {
-    embed = EPHY_EMBED (g_object_new (EPHY_TYPE_EMBED, NULL));
+    GtkWidget *web_view;
+
+    if (related_view)
+      web_view = ephy_web_view_new_with_related_view (related_view);
+    else
+      web_view = ephy_web_view_new ();
+
+    embed = EPHY_EMBED (g_object_new (EPHY_TYPE_EMBED, "web-view", web_view, NULL));
     g_assert (embed != NULL);
     gtk_widget_show (GTK_WIDGET (embed));
 
     ephy_embed_container_add_child (EPHY_EMBED_CONTAINER (window), embed, position, jump_to);
   }
 
-  if (copy_history && previous_embed != NULL) {
-    ephy_web_view_copy_back_history (ephy_embed_get_web_view (previous_embed),
-                                     ephy_embed_get_web_view (embed));
-  }
-
   ephy_gui_window_update_user_time (GTK_WIDGET (window), user_time);
 
   if ((flags & EPHY_NEW_TAB_DONT_SHOW_WINDOW) == 0 &&
@@ -814,7 +815,7 @@ ephy_shell_new_tab (EphyShell *shell,
   EphyEmbed *embed;
   WebKitURIRequest *request = url ? webkit_uri_request_new (url) : NULL;
 
-  embed = ephy_shell_new_tab_full (shell, parent_window,
+  embed = ephy_shell_new_tab_full (shell, NULL, parent_window,
                                    previous_embed, request, flags,
                                    0);
 
@@ -1126,6 +1127,7 @@ ephy_shell_open_uris_idle (OpenURIsData *data)
   }
 
   embed = ephy_shell_new_tab_full (ephy_shell_get_default (),
+                                   NULL,
                                    data->window,
                                    data->previous_embed,
                                    request,
