@@ -150,84 +150,68 @@ popup_cmd_copy_link_address (GtkAction *action,
 	}
 }
 
-static void
-save_property_url_to_destination (EphyWindow *window,
-				  const char *location,
-				  const char *destination)
+static gboolean
+cancel_download_idle_cb (EphyDownload *download)
 {
-	EphyDownload *download;
+	ephy_download_cancel (download);
 
-	download = ephy_download_new_for_uri (location, GTK_WINDOW (window));
-
-	if (destination)
-		ephy_download_set_destination_uri (download, destination);
-	else
-		ephy_download_set_auto_destination (download);
-
-	ephy_download_start (download);
+	return FALSE;
 }
 
 static void
-response_cb (GtkDialog *dialog,
-	     int response_id,
-	     char *location)
+filename_suggested_cb (EphyDownload *download,
+		       const char *suggested_filename,
+		       char *title)
 {
-	if (response_id == GTK_RESPONSE_ACCEPT)
+	EphyWindow *window;
+	EphyFileChooser *dialog;
+	char *sanitized_filename;
+
+	window = EPHY_WINDOW (ephy_download_get_window (download));
+
+	dialog = ephy_file_chooser_new (title,
+					GTK_WIDGET (window),
+					GTK_FILE_CHOOSER_ACTION_SAVE,
+					EPHY_FILE_FILTER_NONE);
+	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
+
+	sanitized_filename = ephy_sanitize_filename (g_strdup (suggested_filename));
+	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), sanitized_filename);
+	g_free (sanitized_filename);
+
+	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
 	{
 		char *uri;
-		GtkWindow *window;
-
-		window = gtk_window_get_transient_for (GTK_WINDOW (dialog));
 
 		uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
-		save_property_url_to_destination (EPHY_WINDOW (window), location, uri);
+		ephy_download_set_destination_uri (download, uri);
 		g_free (uri);
-	}
 
-	g_free (location);
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-}
-
-static char *
-get_suggested_filename (EphyWebView *view)
-{
-	char *suggested_filename = NULL;
-	const char *mimetype;
-	WebKitURIResponse *response;
-	WebKitWebResource *web_resource;
-
-	web_resource = webkit_web_view_get_main_resource (WEBKIT_WEB_VIEW (view));
-	response = webkit_web_resource_get_response (web_resource);
-	mimetype = webkit_uri_response_get_mime_type (response);
-
-	if ((g_ascii_strncasecmp (mimetype, "text/html", 9)) == 0)
-	{
-		/* Web Title will be used as suggested filename */
-		suggested_filename = g_strconcat (ephy_web_view_get_title (view), ".mhtml", NULL);
+		ephy_window_add_download (window, download);
 	}
 	else
 	{
-		suggested_filename = g_strdup (webkit_uri_response_get_suggested_filename (response));
-		if (!suggested_filename)
-		{
-			SoupURI *soup_uri = soup_uri_new (webkit_web_resource_get_uri (web_resource));
-			suggested_filename = g_path_get_basename (soup_uri->path);
-			soup_uri_free (soup_uri);
-		}
+		g_idle_add_full (G_PRIORITY_DEFAULT,
+				 (GSourceFunc)cancel_download_idle_cb,
+				 g_object_ref (download),
+				 g_object_unref);
 	}
 
-	return suggested_filename;
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+	g_free (title);
+
+	g_object_unref (download);
 }
 
 static void
-save_property_url (GtkAction *action,
-		   const char *title,
+save_property_url (const char *title,
 		   EphyWindow *window,
-		   gboolean ask_dest,
 		   const char *property)
 {
 	EphyEmbedEvent *event;
 	const char *location;
+	EphyDownload *download;
 	GValue value = { 0, };
 
 	event = ephy_window_get_context_event (window);
@@ -235,50 +219,25 @@ save_property_url (GtkAction *action,
 
 	ephy_embed_event_get_property (event, property, &value);
 	location = g_value_get_string (&value);
+	download = ephy_download_new_for_uri (location, GTK_WINDOW (window));
+	g_signal_connect (download, "filename-suggested",
+			  G_CALLBACK (filename_suggested_cb),
+			  g_strdup (title));
 
-	if (ask_dest)
-	{
-		EphyFileChooser *dialog;
-		EphyEmbed *embed;
-		EphyWebView *view;
-		char *suggested_filename;
-
-		embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
-		view = ephy_embed_get_web_view (embed);
-
-		dialog = ephy_file_chooser_new (title, GTK_WIDGET (window),
-						GTK_FILE_CHOOSER_ACTION_SAVE,
-						EPHY_FILE_FILTER_NONE);
-
-		suggested_filename = ephy_sanitize_filename (get_suggested_filename (view));
-		gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), suggested_filename);
-		g_free (suggested_filename);
-
-		gtk_file_chooser_set_do_overwrite_confirmation
-				(GTK_FILE_CHOOSER (dialog), TRUE);
-		g_signal_connect (dialog, "response",
-				  G_CALLBACK (response_cb), g_strdup (location));
-		gtk_widget_show (GTK_WIDGET (dialog));
-	}
-	else
-	{
-		save_property_url_to_destination (window, location, NULL);
-	}
+	g_value_unset (&value);
 }
 
 void
 popup_cmd_download_link_as (GtkAction *action,
 			    EphyWindow *window)
 {
-	save_property_url (action, _("Save Link As"), window, 
-			   TRUE, "link-uri");
+	save_property_url (_("Save Link As"), window, "link-uri");
 }
 void
 popup_cmd_save_image_as (GtkAction *action,
 			 EphyWindow *window)
 {
-	save_property_url (action, _("Save Image As"),
-			   window, TRUE, "image-uri");
+	save_property_url (_("Save Image As"), window, "image-uri");
 }
 
 static void
@@ -318,12 +277,12 @@ popup_cmd_set_image_as_background (GtkAction *action,
 
 	ephy_download_set_destination_uri (download, dest_uri);
 	ephy_download_set_action (download, EPHY_DOWNLOAD_ACTION_DO_NOTHING);
+	ephy_window_add_download (window, download);
+	g_object_unref (download);
 
 	g_signal_connect (download, "completed",
 			  G_CALLBACK (background_download_completed), window);
 
-	ephy_download_start (download);
-
 	g_value_unset (&value);
 	g_free (base);
 	g_free (base_converted);
