@@ -84,7 +84,7 @@ fixture_setup (Fixture *fixture, gconstpointer data)
   char *tmp_filename;
   char *dest_file;
 
-  tmp_filename = ephy_file_tmp_filename ("ephy-download-XXXXXX", NULL);
+  tmp_filename = ephy_file_tmp_filename (".ephy-download-XXXXXX", NULL);
   dest_file = g_build_filename (ephy_file_tmp_dir (), tmp_filename, NULL);
 
   fixture->source = get_uri_for_path ("/default");
